from arches.app.utils.search_thumbnail_fetcher_factory import SearchThumbnailFetcherFactory
from arches.app.utils.search_thumbnail_fetcher import SearchThumbnailFetcher
import requests
from django.utils.translation import get_language


@SearchThumbnailFetcherFactory.register("707cbd78-ca7a-11e9-990b-a4d18cec433a")
class DigitalResourceThumbnailFetcher(SearchThumbnailFetcher):
    def get_thumbnail(self, retrieve):
        DIGITAL_RESOURCE_TYPE_NODEGROUP_ID = "09c1778a-ca7b-11e9-860b-a4d18cec433a"
        IIIF_MANIFEST_TYPE = "305c62f0-7e3d-4d52-a210-b451491e6100"
        MANIFEST_NODEGROUP_ID = "56f8e26e-ca7c-11e9-9aa3-a4d18cec433a"
        MANIFEST_URL_NODE_ID = "56f8e9bd-ca7c-11e9-b578-a4d18cec433a"
        self.resource.load_tiles()
        digital_resource_type_tile = next(
            (tile for tile in self.resource.tiles if str(tile.nodegroup_id) == DIGITAL_RESOURCE_TYPE_NODEGROUP_ID), None
        )
        if digital_resource_type_tile and IIIF_MANIFEST_TYPE in digital_resource_type_tile.data[DIGITAL_RESOURCE_TYPE_NODEGROUP_ID]:
            manifest_tile = next((tile for tile in self.resource.tiles if str(tile.nodegroup_id) == MANIFEST_NODEGROUP_ID), None)
            manifest_url = manifest_tile.data[MANIFEST_URL_NODE_ID][get_language()]["value"]
            response = requests.get(manifest_url)
            try:
                response_json = response.json()
                if "thumbnail" in response_json:
                    image_url = response_json["thumbnail"]["@id"]
            except ValueError:
                image_url = manifest_url.replace("full/full", "full/!200,200")

            if retrieve:
                response = requests.get(image_url, allow_redirects=True)
                response.raw.decode_content = True
                if response.ok:
                    return (response.content, response.headers["Content-Type"])
            else:
                response = requests.head(image_url, allow_redirects=True)
                if response.ok:
                    return ()
        return None



import json
from django.conf import settings
from django.http import JsonResponse, HttpResponse
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from arches.app.views.base import BaseManagerView
from arches.app.utils.decorators import can_edit_resource_instance
import boto3
from botocore.config import Config

KEY_BASE = settings.UPLOADED_FILES_DIR


class S3View(BaseManagerView):
    def __init__(self):
        self.config = Config(region_name=settings.AWS_S3_REGION_NAME, s3={"signature_version": "s3v4"})


@method_decorator(can_edit_resource_instance, name="dispatch")
class S3MultipartUploaderView(S3View):
    """S3 Multipart uploader chunks files to allow for parallel uploads to S3"""

    def __init__(self):
        super().__init__()

    def options(self, request):
        response = HttpResponse()
        response.headers["access-control-allow-headers"] = "x-csrftoken,accept,content-type,uppy-auth-token,location"
        return response

    def post(self, request):
        try:
            storage_bucket = settings.AWS_STORAGE_BUCKET_NAME
        except AttributeError:
            raise Exception(_("Django storages for AWS not configured"))

        json_body = json.loads(request.body)
        file_name = json_body["filename"]
        if file_name and file_name.startswith(KEY_BASE):
            key = file_name
        else:
            key = KEY_BASE + file_name

        response_object = {}
        s3 = boto3.client("s3", config=self.config)
        resp = s3.create_multipart_upload(
            Bucket=storage_bucket,
            Key=key,
            ContentType=json_body["type"],
            Metadata=json_body["metadata"],
        )
        response_object["key"] = resp["Key"]
        response_object["uploadId"] = resp["UploadId"]
        return JsonResponse(response_object)


@method_decorator(can_edit_resource_instance, name="dispatch")
class S3MultipartUploadManagerView(S3View):
    """Returns all of the parts of a given upload id"""

    def __init__(self):
        super().__init__()

    def get(self, request, uploadid):
        try:
            storage_bucket = settings.AWS_STORAGE_BUCKET_NAME
        except AttributeError:
            raise Exception(_("Django storages for AWS not configured"))

        parts = []
        key = request.GET.get("key", "")

        def get_parts(client, uploadId, partNumber):
            response = client.list_parts(Bucket=settings.AWS_STORAGE_BUCKET_NAME, Key=key, UploadId=uploadId, PartNumberMarker=partNumber)
            if "Parts" in response:
                parts = parts + response["Parts"]
            if response["IsTruncated"]:
                get_parts(client, uploadId, response["NextPartNumberMarker"])

        s3 = boto3.client("s3", config=self.config)
        get_parts(s3, uploadid, 0)
        return JsonResponse(parts, safe=False)

    def delete(self, request, uploadid):
        try:
            storage_bucket = settings.AWS_STORAGE_BUCKET_NAME
        except AttributeError:
            raise Exception(_("Django storages for AWS not configured"))

        s3 = boto3.client("s3", config=self.config)
        key = request.GET.get("key", "")

        s3.abort_multipart_upload(storage_bucket, key, uploadid)

        return JsonResponse({})


@method_decorator(can_edit_resource_instance, name="dispatch")
class S3BatchSignView(S3View):
    """generates a batch of presigned urls for a group of part numbers"""

    def __init__(self):
        super().__init__()

    def get(self, request, uploadid):
        try:
            storage_bucket = settings.AWS_STORAGE_BUCKET_NAME
        except AttributeError:
            raise Exception(_("Django storages for AWS not configured"))
        key = request.GET.get("key", "")
        part_numbers = [int(x) for x in request.GET.get("partNumbers").split(",")]
        s3 = boto3.client("s3", config=self.config)
        urls = {}
        urls["presignedUrls"] = [None] * part_numbers[0]
        for part in part_numbers:
            urls["presignedUrls"].append(
                s3.generate_presigned_url(
                    "upload_part",
                    {
                        "Key": key,
                        "Bucket": storage_bucket,
                        "PartNumber": int(part),
                        "Body": "",
                        "UploadId": uploadid,
                    },
                    300,
                )
            )
        return JsonResponse(urls, safe=False)


@method_decorator(can_edit_resource_instance, name="dispatch")
class S3UploadView(S3View):
    """Generates a single presigned URL to be used in a post to S3 (for small files)"""

    def __init__(self):
        super().__init__()

    def get(self, request):
        try:
            storage_bucket = settings.AWS_STORAGE_BUCKET_NAME
        except AttributeError:
            raise Exception(_("Django storages for AWS not configured"))
        s3 = boto3.client("s3", config=self.config)

        file_name = request.GET.get("filename")

        if file_name.startswith(KEY_BASE):
            key = file_name
        else:
            key = KEY_BASE + "/" + file_name

        fields = {}
        response = s3.generate_presigned_post(
            storage_bucket,
            key,
            fields,
            ExpiresIn=300,
        )
        return JsonResponse({"method": "post", "url": response["url"], "fields": response["fields"], "expires": 300}, safe=False)


@method_decorator(can_edit_resource_instance, name="dispatch")
class S3UploadPartView(S3View):
    """Generates a presigned URL for a single part of a multipart upload"""

    def __init__(self):
        super().__init__()

    def get(self, request, uploadid, partnumber):
        try:
            storage_bucket = settings.AWS_STORAGE_BUCKET_NAME
        except AttributeError:
            raise Exception(_("Django storages for AWS not configured"))
        s3 = boto3.client("s3", config=self.config)
        key = request.GET.get("key", "")
        url = s3.generate_presigned_url(
            "upload_part",
            {
                "Key": key,
                "Bucket": storage_bucket,
                "PartNumber": int(partnumber),
                "UploadId": uploadid,
            },
            300,
        )
        return JsonResponse({"url": url, "expires": 300}, safe=False)


@method_decorator(can_edit_resource_instance, name="dispatch")
class S3CompleteUploadView(S3View):
    """Finalizes a multipart upload in s3"""

    def __init__(self):
        super().__init__()

    def post(self, request, uploadid):
        try:
            storage_bucket = settings.AWS_STORAGE_BUCKET_NAME
        except AttributeError:
            raise Exception(_("Django storages for AWS not configured"))
        key = request.GET.get("key", "")
        s3 = boto3.client("s3", config=self.config)
        response = s3.complete_multipart_upload(
            Bucket=storage_bucket,
            Key=key,
            UploadId=uploadid,
            MultipartUpload={"Parts": json.loads(request.body.decode("utf-8"))["parts"]},
        )
        return JsonResponse({"location": response["Location"]})



import datetime
import logging
from io import StringIO
from io import BytesIO
import json
from django.contrib.auth.models import User
from django.core.files import File
from django.http import HttpRequest
from django.utils.translation import gettext as _
from django.views.generic import View
from arches.app.models import models
from arches.app.utils.response import JSONResponse, JSONErrorResponse
import arches.app.utils.zip as zip_utils
import arches.app.utils.task_management as task_management
import arches_for_science.tasks as tasks
from arches_templating.views.template import TemplateView

logger = logging.getLogger(__name__)


class FileDownloader(View):
    def post(self, request):
        json_data = json.loads(request.body)

        # get reports and save them
        templates = json_data.pop("templates")
        generated_reports = []
        for template in templates:
            request.POST = request.POST.copy()
            json_data["templateId"] = template["templateId"]
            json_data["filename"] = template["filename"]
            request._body = json.dumps(json_data)  # TODO: there should be a better way
            report_template = TemplateView()
            response = report_template.post(request, template["templateId"])

            name = response.headers["Content-Disposition"].split("=")[1]  # TODO: need more robust way to do this
            content = response.content
            f = BytesIO(content)
            file = File(f)

            report_file = models.TempFile()
            report_file.source = "project-report-workflow"
            report_file.path.save(name, file)

            generated_reports.append({"name": name, "fileid": report_file.fileid})

        # get attached files and zip them
        files = json_data["files"]
        screenshots = [{"fileid": i["fileId"], "name": i["imageName"]} for i in json_data["annotationScreenshots"]]
        temp_files = generated_reports + screenshots

        project_name = json_data["projectDetails"][0]["displayname"]
        userid = request.user.id

        if len(files) + len(temp_files) == 0:
            message = _(
                "The report(s) are submitted for download. When completed, the bell icon at the top of the page will update with links to download your files."
            )
            return JSONResponse({"success": True, "message": message})

        if task_management.check_if_celery_available():
            result = tasks.download_project_files_task.apply_async(
                (userid, files, temp_files, project_name),
            )
            message = _(
                "The report(s) and {} additional file(s) are submitted for download. When completed, the bell icon at the top of the page will update with links to download your files."
            ).format(len(files) + len(screenshots))
            return JSONResponse({"success": True, "message": message})
        else:
            response = {
                "title": _("Error"),
                "message": _("Celery must be running to download files. Check with your Arches administrator for help."),
            }
            return JSONErrorResponse(content=response)

    def create_download_zipfile(self, user, files, temp_files, project_name):
        file_ids = [file["fileid"] for file in files]
        file_objects = list(models.File.objects.filter(pk__in=file_ids))

        temp_file_ids = [temp_file["fileid"] for temp_file in temp_files]
        temp_file_objects = list(models.TempFile.objects.filter(pk__in=temp_file_ids))

        all_files = files + temp_files
        all_file_objects = file_objects + temp_file_objects

        for file in all_files:
            for file_object in all_file_objects:
                if str(file_object.fileid) == str(file["fileid"]):
                    file["file"] = file_object.path
        download_files = []
        skipped_files = []
        size_limit = 104857600  # 100MByte
        for file in all_files:
            try:
                if file["file"].size >= size_limit:
                    skipped_files.append({"name": file["name"], "fileid": file["fileid"]})
                else:
                    download_files.append({"name": file["name"], "downloadfile": file["file"]})
            except:
                logger.warning(_("Unable to locate {}".format(file["name"])))

        if len(download_files) > 0:
            zip_stream = zip_utils.create_zip_file(download_files, "downloadfile")
            now = datetime.datetime.now().isoformat()
            name = f"{project_name}_{now}.zip"
            search_history_obj = models.SearchExportHistory(user=user, numberofinstances=len(files))
            f = BytesIO(zip_stream)
            download = File(f)
            search_history_obj.downloadfile.save(name, download)
            return search_history_obj.searchexportid, skipped_files
        else:
            return None, skipped_files



from django.apps import AppConfig

from arches.settings_utils import generate_frontend_configuration


class ArchesForScienceConfig(AppConfig):
    name = "arches_for_science"
    verbose_name = "Arches for Science"
    is_arches_application = True

    def ready(self):
        generate_frontend_configuration()
        from . import signals



from django.conf import settings
from django.urls import include, path, re_path
from django.conf.urls.i18n import i18n_patterns
from django.conf.urls.static import static
from arches_for_science.views.renderer_config import RendererConfigView, RendererView
from arches_for_science.views.workflows.upload_dataset.format_render_map import FormatRenderMap
from arches_for_science.views.workflows.upload_dataset.update_file_format import UpdateFileFormat
from arches_for_science.views.workflows.upload_dataset.select_dataset_files_step import SelectDatasetFilesStep
from arches_for_science.views.download_project_files import FileDownloader
from arches_for_science.views.physical_thing_search import PhysicalThingSearchView
from arches_for_science.views.physical_things_in_set import PhysicalThingSetView
from arches_for_science.views.s3 import (
    S3MultipartUploadManagerView,
    S3MultipartUploaderView,
    S3BatchSignView,
    S3CompleteUploadView,
    S3UploadPartView,
    S3UploadView,
)
from arches_for_science.views.update_resource_list import UpdateResourceListView
from arches_for_science.views.analysis_area_and_sample_taking import (
    SaveAnalysisAreaView,
    SaveSampleAreaView,
    DeleteSampleAreaView,
    DeleteAnalysisAreaView,
    GetLockedStatus,
)
from arches_for_science.views.digital_resources_by_object_parts import DigitalResourcesByObjectParts
from arches_for_science.views.instrument_info_step import InstrumentInfoStepFormSaveView
from arches_for_science.views.manifest_x_canvas import ManifestXCanvasView

uuid_regex = settings.UUID_REGEX

urlpatterns = [
    re_path(r"^physical-thing-search-results", PhysicalThingSearchView.as_view(), name="physical-thing-search-results"),
    re_path(r"^physical-things-in-set", PhysicalThingSetView.as_view(), name="physical_things_set"),
    re_path(
        r"^digital-resources-by-object-parts/(?P<resourceid>%s)$" % uuid_regex,
        DigitalResourcesByObjectParts.as_view(),
        name="digital-resources-by-object-parts",
    ),
    re_path(
        r"^workflows/upload-dataset-workflow/select-dataset-files-step",
        SelectDatasetFilesStep.as_view(),
        name="upload_dataset_select_dataset_files_step",
    ),
    re_path(
        r"^workflows/upload-dataset-workflow/file-renderer/(?P<tileid>%s)$" % uuid_regex,
        UpdateFileFormat.as_view(),
        name="upload_dataset_file_renderer",
    ),
    re_path(
        r"^workflows/upload-dataset-workflow/get-format-renderer/(?P<format>[0-9a-zA-Z_\-./]*)$",
        FormatRenderMap.as_view(),
        name="format_render_map",
    ),
    re_path(r"^updateresourcelist", UpdateResourceListView.as_view(), name="updateresourcelist"),
    re_path(r"^uppy/s3/multipart/(?P<uploadid>[^\/]+)/complete$", S3CompleteUploadView.as_view(), name="s3_multipart_upload_complete"),
    re_path(r"^uppy/s3/multipart/(?P<uploadid>[^\/]+)/batch", S3BatchSignView.as_view(), name="s3_multipart_batch_sign"),
    re_path(r"^uppy/s3/multipart/(?P<uploadid>[^\/]+)/(?P<partnumber>\d+)", S3UploadPartView.as_view(), name="s3_multipart_upload_part"),
    re_path(r"^uppy/s3/multipart/(?P<uploadid>[^\/]+)", S3MultipartUploadManagerView.as_view(), name="s3_multipart_upload"),
    re_path(r"^uppy/s3/multipart$", S3MultipartUploaderView.as_view(), name="s3_multipart_upload"),
    re_path(r"^uppy/s3/params", S3UploadView.as_view(), name="s3_upload"),
    re_path(r"^instrument-info-form-save", InstrumentInfoStepFormSaveView.as_view(), name="instrument-info-form-save"),
    re_path(r"^saveanalysisarea", SaveAnalysisAreaView.as_view(), name="saveanalysisarea"),
    re_path(r"^savesamplearea", SaveSampleAreaView.as_view(), name="savesamplearea"),
    re_path(r"^deletesamplearea", DeleteSampleAreaView.as_view(), name="deletesamplearea"),
    re_path(r"^deleteanalysisarea", DeleteAnalysisAreaView.as_view(), name="deleteanalysisarea"),
    re_path(r"^analysisarealocked", GetLockedStatus.as_view(), name="analysisarealocked"),
    re_path(r"^download_project_files", FileDownloader.as_view(), name="download_project_files"),
    re_path(r"^renderer/(?P<renderer_id>[^\/]+)", RendererView.as_view(), name="renderer"),
    re_path(r"^renderer_config/(?P<renderer_config_id>[^\/]+)", RendererConfigView.as_view(), name="renderer_config"),
    re_path(r"^renderer_config/", RendererConfigView.as_view(), name="renderer_config"),
    re_path(r"^manifest_x_canvas/", ManifestXCanvasView.as_view(), name="manifest_x_canvas"),
    path("reports/", include("arches_templating.urls")),
    path("", include("arches.urls")),
] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

# Only handle i18n routing in active project. This will still handle the routes provided by Arches core and Arches applications,
# but handling i18n routes in multiple places causes application errors.
if settings.ROOT_URLCONF == __name__:
    if settings.SHOW_LANGUAGE_SWITCH is True:
        urlpatterns = i18n_patterns(*urlpatterns)

    urlpatterns.append(path("i18n/", include("django.conf.urls.i18n")))



from django.views.generic import View
from arches.app.utils.response import JSONResponse
from arches.app.models import models
from arches.app.views.search import search_results
from django.http import HttpRequest
import json


class PhysicalThingSetView(View):
    def get(self, request):
        # via an id for a set, returns list of phys things and stuff necessary
        set_resourceid = (
            None if request.GET.get("resourceid") == "" or request.GET.get("resourceid") is None else (request.GET.get("resourceid"))
        )
        nodegroupid = request.GET.get("nodegroupid")
        nodeid = request.GET.get("nodeid")
        related = []
        results = []
        tiles = models.TileModel.objects.filter(resourceinstance_id=set_resourceid).filter(nodegroup_id=nodegroupid)
        if len(tiles) > 0:
            tile = tiles[0]
            related = tile.data[nodeid]
        for related_resource_item in related:
            search_request = HttpRequest()
            search_request.user = request.user
            search_request.GET["id"] = related_resource_item["resourceId"]
            result = json.loads(search_results(search_request).content)
            results.append(result["results"]["hits"]["hits"][0])

        return JSONResponse({"items": results})



from arches.app.functions.primary_descriptors import AbstractPrimaryDescriptorsFunction
from arches.app.models.system_settings import settings

from django.utils.translation import get_language, gettext as _

# This duplicates the configuration declared in migration 0004,
# but on first package load, the function will be re-registered, because
# the .py file has not yet been placed in the destination folder.
# Re-registration will overwrite whatever the migration inserted.
details = {
    "functionid": "00b2d15a-fda0-4578-b79a-784e4138664b",
    "name": "Multi-card Resource Descriptor",
    "type": "primarydescriptors",
    "description": "Configure the name, description, and map popup of a resource",
    "defaultconfig": {
        "descriptor_types": {
            "name": {
                "nodegroup_id": "",
                "string_template": "",
            },
            "map_popup": {
                "nodegroup_id": "",
                "string_template": "",
            },
            "description": {
                "nodegroup_id": "",
                "string_template": "",
            },
        }
    },
    "classname": "MulticardResourceDescriptor",
    "component": "views/components/functions/multicard-resource-descriptor",
}


class MulticardResourceDescriptor(AbstractPrimaryDescriptorsFunction):
    """Implemented in the database via triggers on tiles table.

    This implementation just fetches the calculated result from the db."""

    def get_primary_descriptor_from_nodes(self, resource, config, context=None, descriptor=None):
        resource.refresh_from_db(fields={"name", "descriptors"})
        resource.get_descriptor_language(context)

        result = ""
        requested_language = context.get("language", None) if context else None

        lookup_language = requested_language or get_language() or settings.LANGUAGE_CODE
        try:
            result = resource.descriptors[lookup_language][descriptor]
        except KeyError:
            pass

        if result.strip() == "":
            result = _("Undefined")

        return result



from arches.app.utils.search_thumbnail_fetcher_factory import SearchThumbnailFetcherFactory
from arches.app.utils.search_thumbnail_fetcher import SearchThumbnailFetcher


@SearchThumbnailFetcherFactory.register("9519cb4f-b25b-11e9-8c7b-a4d18cec433a")
class PhysicalThingThumbnailFetcher(SearchThumbnailFetcher):
    def get_thumbnail(self, retrieve):
        PRIMARY_MANIFEST_NODEGROUP_ID = "8a4ad932-8d59-11eb-a9c4-faffc265b501"
        DIGITAL_RESOURCE_RELATIONSHIP_NODE_ID = "a298ee52-8d59-11eb-a9c4-faffc265b501"
        self.resource.load_tiles()
        manifest_tile = next((tile for tile in self.resource.tiles if str(tile.nodegroup_id) == PRIMARY_MANIFEST_NODEGROUP_ID), None)
        if manifest_tile:
            digital_resource_id = manifest_tile.data[DIGITAL_RESOURCE_RELATIONSHIP_NODE_ID][0]["resourceId"]
            factory = SearchThumbnailFetcherFactory()
            fetcher = factory.create_thumbnail_fetcher(digital_resource_id)
            return fetcher.get_thumbnail(retrieve)
        else:
            return None



from io import FileIO
from mimetypes import MimeTypes
from django.views.generic import View
from django.db import transaction
import os
import io
from arches.app.datatypes.datatypes import StringDataType
from arches.app.utils.response import JSONResponse
from arches.app.views.tile import TileData
from arches.app.models.tile import Tile
from arches.app.models.resource import Resource
from arches.app.models.models import Node
from arches.app.utils.betterJSONSerializer import JSONSerializer, JSONDeserializer
import json
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.http import HttpRequest
from django.conf import settings


class SelectDatasetFilesStep(View):
    def post(self, request):
        recorded_value_node_id = "dd596aae-c457-11e9-956b-a4d18cec433a"
        digital_reference_node_id = "a298ee52-8d59-11eb-a9c4-faffc265b501"
        digital_reference_node_group_id = "8a4ad932-8d59-11eb-a9c4-faffc265b501"
        dataset_name_node_group_id = "d2fdae3d-ca7a-11e9-ad84-a4d18cec433a"
        dataset_name_node_id = "d2fdc2fa-ca7a-11e9-8ffb-a4d18cec433a"
        dataset_file_node_group_id = "7c486328-d380-11e9-b88e-a4d18cec433a"
        dataset_file_node_id = "7c486328-d380-11e9-b88e-a4d18cec433a"

        # relationship valueids
        recorded_value_of_observation = "96a4e8f5-6214-4af3-b4cc-b79c206d00f0"
        output_of = "9566b743-b23e-41df-bddb-af75f22edcc5"
        digital_source = "be3f33e9-216d-4355-8766-aced1e95616c"
        digital_source_for = "ff6a0510-6c91-4c45-8c67-dbbcf8d7d7fa"

        transaction_id = request.POST.get("transaction_id")
        observation_id = request.POST.get("observation_id")
        posted_dataset = request.POST.get("dataset")
        observation_ref_tile = request.POST.get("observation_ref_tile", None)

        with transaction.atomic():
            dataset = JSONDeserializer().deserialize(posted_dataset)
            tile_id = dataset.get("tileId", None)
            part_resource_id = dataset.get("partResourceId", None)
            dataset_resource_id = dataset.get("resourceInstanceId", None)

            # create a dataset (digital resource) and a name tile for it.
            if not dataset_resource_id:
                graphid = Node.objects.filter(nodegroup=dataset_name_node_group_id)[0].graph_id
                dataset_resource = Resource()
                dataset_resource.graph_id = graphid
                dataset_resource.save(user=request.user, transaction_id=transaction_id)

                dataset_name_tile = Tile().get_blank_tile_from_nodegroup_id(dataset_name_node_group_id)
                dataset_name_tile.resourceinstance_id = dataset_resource.resourceinstanceid
            else:
                dataset_resource = Resource.objects.get(pk=dataset_resource_id)
                dataset_name_tile = Tile.objects.get(pk=tile_id)

            string_data_type = StringDataType()
            dataset_name_tile.data[dataset_name_node_id] = string_data_type.transform_value_for_tile(dataset.get("name"))
            dataset_name_tile.save(transaction_id=transaction_id)
            dataset_resource.load_tiles()

            if not dataset_resource_id:
                # save dataset's relationship with part resource ID.
                digital_reference_tile = Tile().get_blank_tile_from_nodegroup_id(digital_reference_node_group_id)
                digital_reference_tile.resourceinstance_id = part_resource_id
                digital_reference_tile.data[digital_reference_node_id] = [
                    {
                        "resourceId": str(dataset_resource.resourceinstanceid),
                        "ontologyProperty": digital_source,
                        "inverseOntologyProperty": digital_source_for,
                    }
                ]
                digital_reference_tile.save(user=request.user, transaction_id=transaction_id)

            # create observation cross references
            if observation_ref_tile is not None:
                observation_reference_tile = Tile.objects.get(tileid=observation_ref_tile)
                relationships = observation_reference_tile.data[recorded_value_node_id]
                part_relationships = [item for item in relationships if item["partResource"] == part_resource_id]
                if len(part_relationships) == 0:
                    observation_reference_tile.data[recorded_value_node_id].append(
                        {
                            "resourceId": str(dataset_resource.resourceinstanceid),
                            "ontologyProperty": recorded_value_of_observation,
                            "inverseOntologyProperty": output_of,
                            "partResource": part_resource_id,
                        }
                    )
            else:
                observation_reference_tile = Tile().get_blank_tile_from_nodegroup_id(recorded_value_node_id)
                observation_reference_tile.resourceinstance_id = observation_id
                observation_reference_tile.data[recorded_value_node_id] = [
                    {
                        "resourceId": str(dataset_resource.resourceinstanceid),
                        "ontologyProperty": recorded_value_of_observation,
                        "inverseOntologyProperty": output_of,
                        "partResource": part_resource_id,
                    }
                ]

            observation_reference_tile.save(user=request.user, transaction_id=transaction_id)

            # save files associated with digital resource
            dataset_files = request.FILES.getlist("file-list_{}_preloaded".format(dataset_file_node_id), []) + request.FILES.getlist(
                "file-list_{}".format(dataset_file_node_id), []
            )
            file_data_list = request.POST.getlist("file-list_{}_data".format(dataset_file_node_id), None)
            file_data_list = [JSONDeserializer().deserialize(fd) for fd in file_data_list]
            new_files = []
            for file in dataset_files:
                file_data = next((fd for fd in file_data_list if fd.get("name") == file.name), None)

                new_files.append((file_data, file))

            removed_files = []
            for file in new_files:
                if file[0] is not None:
                    file_data = file[0]
                    file_content = file[1]

                    tiles = (tile for tile in dataset_resource.tiles if str(tile.nodegroup_id) == dataset_file_node_group_id)
                    # delete/replace tiles for files that share the same name
                    for tile in tiles:
                        if tile.data[dataset_file_node_id][0]["name"] == file_data["name"]:
                            removed_files.append({"name": file_data["name"], "tileid": tile.tileid})
                            Tile.objects.get(pk=tile.tileid).delete(request=request)
                    dataset_resource.load_tiles()

                    split_file_mime = file_data["type"].split("/")
                    if "image" in split_file_mime:
                        file_data["renderer"] = next(
                            (renderer["id"] for renderer in settings.RENDERERS if renderer["name"] == "imagereader"), None
                        )
                    elif "pdf" in split_file_mime:
                        file_data["renderer"] = next(
                            (renderer["id"] for renderer in settings.RENDERERS if renderer["name"] == "pdfreader"), None
                        )
                    elif file_data["name"].split(".")[-1] in settings.XY_TEXT_FILE_FORMATS:  # instrument was given by zip file name
                        file_data["renderer"] = next(
                            (renderer["id"] for renderer in settings.RENDERERS if renderer["name"] == "xy-reader"), None
                        )
                        file_data["type"] = "text/plain"

                    # file has not been uploaded
                    dataset_file_tile = Tile().get_blank_tile_from_nodegroup_id(dataset_file_node_group_id)
                    dataset_file_tile.resourceinstance_id = str(dataset_resource.resourceinstanceid)
                    dataset_file_tile.data[dataset_file_node_id] = [file[0]]
                    dataset_file_tile.tileid = ""
                    dataset_file_tile.pk = ""
                    new_req = HttpRequest()
                    new_req.method = "POST"
                    new_req.user = request.user
                    new_req.POST["data"] = json.dumps(dataset_file_tile.serialize())
                    new_req.POST["transaction_id"] = transaction_id
                    if len(request.FILES.getlist("file-list_{}".format(dataset_file_node_id), [])) > 0:
                        new_req.FILES["file-list_{}".format(dataset_file_node_id)] = file_content
                    elif len(request.FILES.getlist("file-list_{}_preloaded".format(dataset_file_node_id), [])) > 0:
                        new_req.FILES["file-list_{}_preloaded".format(dataset_file_node_id)] = file_content
                    new_tile = TileData()
                    new_tile.action = "update_tile"

                    raw_response = new_tile.post(new_req)
                    response = json.loads(raw_response.content)

                    # raising the error here will cause the transaction to fail; the resource/tiles will not be saved
                    if raw_response.status_code != 200:
                        raise Exception
                    else:
                        file_data["tileid"] = response["tileid"]
                        file_data["path"] = response["data"][dataset_file_node_id][0]["path"]
            file_response = [
                {
                    "name": f[0]["name"],
                    "path": f[0]["path"],
                    "clientId": f[0]["clientFileId"] if "clientFileId" in f[0] else "",
                    "renderer": f[0]["renderer"] if "renderer" in f[0] else "",
                    "format": f[0].get("format", None),
                    "tileId": f[0]["tileid"],
                }
                for f in new_files
            ]

        return JSONResponse(
            {
                "observationReferenceTileId": str(observation_reference_tile.tileid),
                "datasetResourceId": str(dataset_name_tile.resourceinstance.resourceinstanceid),
                "datasetNameTileId": dataset_name_tile.tileid,
                "removedFiles": removed_files,
                "files": file_response,
            }
        )



from io import FileIO
from mimetypes import MimeTypes
from django.views.generic import View
from django.db import transaction
import os
import io
from arches.app.utils.response import JSONResponse
from arches.app.views.tile import TileData
from arches.app.models.tile import Tile
from arches.app.models.resource import Resource
from arches.app.models.models import Node
from arches.app.utils.betterJSONSerializer import JSONSerializer, JSONDeserializer
import json
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.http import HttpRequest
import zipfile


class FormatRenderMap(View):
    def get(self, request, format):

        renderer_lookup = {
            "fors": "88dccb59-14e3-4445-8f1b-07f0470b38bb",
            "raman": "94fa1720-6773-4f99-b49b-4ea0926b3933",
            "xrf": "31be40ae-dbe6-4f41-9c13-1964d7d17042",
        }

        format_mappings = {
            "bm6": "xrf",
            "b5g": "xrf",
            "bt45": "xrf",
            "bt3": "xrf",
            "b5i": "xrf",
            "bart": "xrf",
            "r785": "raman",
            "r633": "raman",
            "asd": "fors",
        }
        if format in format_mappings:
            return JSONResponse({"renderer": renderer_lookup[format_mappings[format]]})
        else:
            return None



from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from arches.app.models.models import IIIFManifest, ResourceXResource, TileModel
from arches.app.models.resource import Resource
from .models import ManifestXCanvas, ManifestXDigitalResource
import uuid


@receiver(post_save, sender=IIIFManifest)
def create_digital_resources(sender, instance, created, **kwargs):
    from arches_for_science.utils.digital_resource_for_manifest import digital_resources_for_manifest, digital_resources_for_canvases

    digital_resources_for_manifest(instance, created)
    digital_resources_for_canvases(instance)


@receiver(post_delete, sender=IIIFManifest)
def delete_manifest_x_canvas(sender, instance, **kwargs):
    ManifestXCanvas.objects.filter(manifest=instance.manifest["@id"]).delete()
    ManifestXDigitalResource.objects.filter(manifest=instance.manifest["@id"]).delete()


@receiver(post_delete, sender=ResourceXResource)
def ensure_part_tile_is_deleted(sender, instance, **kwargs):
    physical_thing_parts_node_id = uuid.UUID("b240c366-8594-11ea-97eb-acde48001122")
    if instance.nodeid_id == physical_thing_parts_node_id:
        try:
            instance.tileid.delete()
            resourceid = instance.tileid.resourceinstance_id
            resource = Resource.objects.get(pk=resourceid)
            resource.index()
        except TileModel.DoesNotExist:
            pass



import logging
from django.views.generic import View
from arches_for_science.models import RendererConfig
from arches.app.models import models
from arches.app.utils.response import JSONResponse
from django.http import HttpResponseNotFound
from django.db.models import Q
from django.utils.translation import gettext as _
from arches.app.utils.betterJSONSerializer import JSONSerializer, JSONDeserializer

logger = logging.getLogger(__name__)


class RendererView(View):
    def get(self, request, renderer_id=None):
        renderer = {}
        if renderer_id is None:
            return JSONResponse([])  # this should be fixed later to return all renderers; not currently used.
        else:
            renderer_config = RendererConfig.objects.filter(rendererid=renderer_id)

            if renderer_config:
                renderer["configs"] = renderer_config.values()
                return JSONResponse(renderer)
            else:
                return HttpResponseNotFound(_("<h1>Renderers do not exist</h1>"))


class RendererConfigView(View):
    def get(self, request, renderer_config_id=None):
        if renderer_config_id is None:
            return JSONResponse(RendererConfig.objects.all().values())

        else:
            renderer_config = RendererConfig.objects.filter(configid=renderer_config_id)

            if renderer_config:
                return JSONResponse(renderer_config.values())
            else:
                return HttpResponseNotFound(_("<h1>Renderers config does not exist</h1>"))

    def post(self, request, renderer_config_id=None):
        body = JSONDeserializer().deserialize(request.body)
        if renderer_config_id:
            renderer_config = RendererConfig.objects.get(configid=renderer_config_id)
            renderer_config.rendererid = body["rendererId"]
            renderer_config.name = body["name"]
            renderer_config.description = body["description"]
            body.pop("rendererId")
            body.pop("name")
            body.pop("description")
            renderer_config.config = body
            renderer_config.save()
        else:
            renderer_config = RendererConfig.objects.create(rendererid=body["rendererId"], name=body["name"], config=body)

        response_dict = JSONSerializer().serialize(renderer_config)

        return JSONResponse(response_dict)

    def delete(self, request, renderer_config_id):
        file_nodegroup_id = "7c486328-d380-11e9-b88e-a4d18cec433a"
        renderer_config = RendererConfig.objects.get(configid=renderer_config_id)
        query = Q(
            **{
                "nodegroup_id": file_nodegroup_id,
                "data__has_key": file_nodegroup_id,
                f"data__{file_nodegroup_id}__0__rendererConfig": renderer_config_id,
            }
        )

        renderer_used = models.TileModel.objects.filter(query).exists()
        if not renderer_used:
            renderer_config.delete()
            response_dict = {"deleted": JSONSerializer().serialize(renderer_config)}
        else:
            response_dict = {"deleted": False}

        return JSONResponse(response_dict)



"""
Django settings for arches_for_science project.
"""

import os
import arches
import inspect
import semantic_version
from datetime import datetime, timedelta
from django.utils.translation import gettext_lazy as _

try:
    from arches.settings import *
except ImportError:
    pass

APP_NAME = "arches_for_science"
APP_VERSION = semantic_version.Version(major=0, minor=0, patch=0)
APP_ROOT = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))

WEBPACK_LOADER = {
    "DEFAULT": {
        "STATS_FILE": os.path.join(APP_ROOT, "..", "webpack/webpack-stats.json"),
    },
}

DATATYPE_LOCATIONS.append("arches_for_science.datatypes")
FUNCTION_LOCATIONS.append("arches_for_science.functions")
ETL_MODULE_LOCATIONS.append("arches_for_science.etl_modules")
SEARCH_COMPONENT_LOCATIONS.append("arches_for_science.search_components")

LOCALE_PATHS.insert(0, os.path.join(APP_ROOT, "locale"))

FILE_TYPE_CHECKING = "lenient"
FILE_TYPES = [
    "bmp",
    "gif",
    "jpg",
    "jpeg",
    "json",
    "pdf",
    "png",
    "psd",
    "rtf",
    "tif",
    "tiff",
    "xlsx",
    "csv",
    "zip",
]
FILENAME_GENERATOR = "arches.app.utils.storage_filename_generator.generate_filename"
UPLOADED_FILES_DIR = "uploadedfiles"

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "#f56+u&mpoz^s&2qjk!v1j(jc&zr1*ana9z5q=7!246cx-1(iv"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ROOT_URLCONF = "arches_for_science.urls"
ELASTICSEARCH_HOSTS = [{"scheme": "http", "host": "localhost", "port": 9200}]
ELASTICSEARCH_CONNECTION_OPTIONS = {"request_timeout": 30, "verify_certs": False}
# ELASTICSEARCH_CONNECTION_OPTIONS = {"request_timeout": 30, "verify_certs": False, "basic_auth": ("elastic", "E1asticSearchforArche5")}
ELASTICSEARCH_PREFIX = "arches_for_science"

LOAD_DEFAULT_ONTOLOGY = False
LOAD_PACKAGE_ONTOLOGIES = True

# This is the namespace to use for export of data (for RDF/XML for example)
# It must point to the url where you host your site
# Make sure to use a trailing slash
ARCHES_NAMESPACE_FOR_DATA_EXPORT = "http://localhost:8000/"
PUBLIC_SERVER_ADDRESS = "http://localhost:8002/"

DATABASES = {
    "default": {
        "ATOMIC_REQUESTS": False,
        "AUTOCOMMIT": True,
        "CONN_MAX_AGE": 0,
        "ENGINE": "django.contrib.gis.db.backends.postgis",
        "HOST": "localhost",
        "NAME": "arches_for_science",
        "OPTIONS": {
            "options": "-c cursor_tuple_fraction=1",
        },
        "PASSWORD": "postgis",
        "PORT": "5432",
        "POSTGIS_TEMPLATE": "template_postgis",
        "TEST": {"CHARSET": None, "COLLATION": None, "MIRROR": None, "NAME": None},
        "TIME_ZONE": None,
        "USER": "postgres",
    }
}

SEARCH_THUMBNAILS = False

INSTALLED_APPS = (
    "webpack_loader",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.gis",
    "arches",
    "arches.app.models",
    "arches.management",
    "guardian",
    "captcha",
    "revproxy",
    "corsheaders",
    "oauth2_provider",
    "django_celery_results",
    "pgtrigger",
    "arches_templating",
    # "silk",
    "arches_for_science",  # Ensure the project is listed before any other arches applications
)

# Placing this last ensures any templates provided by Arches Applications
# take precedence over core arches templates in arches/app/templates.
INSTALLED_APPS += ("arches.app",)

MIDDLEWARE = [
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    #'arches.app.utils.middleware.TokenMiddleware',
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "arches.app.utils.middleware.ModifyAuthorizationHeader",
    "oauth2_provider.middleware.OAuth2TokenMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    # "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "arches.app.utils.middleware.SetAnonymousUser",
    # "silk.middleware.SilkyMiddleware",
]

STATICFILES_DIRS = build_staticfiles_dirs(app_root=APP_ROOT)

TEMPLATES = build_templates_config(
    debug=DEBUG,
    app_root=APP_ROOT,
)

ALLOWED_HOSTS = []

SYSTEM_SETTINGS_LOCAL_PATH = os.path.join(APP_ROOT, "system_settings", "System_Settings.json")
WSGI_APPLICATION = "arches_for_science.wsgi.application"

# URL that handles the media served from MEDIA_ROOT, used for managing stored files.
# It must end in a slash if set to a non-empty value.
MEDIA_URL = "/files/"

# Absolute filesystem path to the directory that will hold user-uploaded files.
MEDIA_ROOT = os.path.join(APP_ROOT)

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = "/static/"

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(APP_ROOT, "staticfiles")

# when hosting Arches under a sub path set this value to the sub path eg : "/{sub_path}/"
FORCE_SCRIPT_NAME = None

RESOURCE_IMPORT_LOG = os.path.join(APP_ROOT, "logs", "resource_import.log")
DEFAULT_RESOURCE_IMPORT_USER = {"username": "admin", "userid": 1}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "console": {
            "format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s",
        },
    },
    "handlers": {
        "file": {
            "level": "WARNING",  # DEBUG, INFO, WARNING, ERROR
            "class": "logging.FileHandler",
            "filename": os.path.join(APP_ROOT, "arches.log"),
            "formatter": "console",
        },
        "console": {"level": "WARNING", "class": "logging.StreamHandler", "formatter": "console"},
    },
    "loggers": {
        "arches": {
            "handlers": ["file", "console"],
            "level": "WARNING",
            "propagate": True,
        },
        "arches_for_science": {
            "handlers": ["file", "console"],
            "level": "WARNING",
            "propagate": True,
        },
        "django.request": {
            "handlers": ["file", "console"],
            "level": "WARNING",
            "propagate": True,
        },
    },
}

# Rate limit for authentication views
# See options (including None or python callables):
# https://django-ratelimit.readthedocs.io/en/stable/rates.html#rates-chapter
RATE_LIMIT = "5/m"

# Sets default max upload size to 15MB
DATA_UPLOAD_MAX_MEMORY_SIZE = 15728640

# Unique session cookie ensures that logins are treated separately for each app
SESSION_COOKIE_NAME = "arches_for_science"

# For more info on configuring your cache: https://docs.djangoproject.com/en/2.2/topics/cache/
CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    },
    "user_permission": {
        "BACKEND": "django.core.cache.backends.db.DatabaseCache",
        "LOCATION": "user_permission_cache",
    },
}

# Hide nodes and cards in a report that have no data
HIDE_EMPTY_NODES_IN_REPORT = False

BYPASS_UNIQUE_CONSTRAINT_TILE_VALIDATION = False
BYPASS_REQUIRED_VALUE_TILE_VALIDATION = False

DATE_IMPORT_EXPORT_FORMAT = "%Y-%m-%d"  # Custom date format for dates imported from and exported to csv

# This is used to indicate whether the data in the CSV and SHP exports should be
# ordered as seen in the resource cards or not.
EXPORT_DATA_FIELDS_IN_CARD_ORDER = False

# Identify the usernames and duration (seconds) for which you want to cache the time wheel
CACHE_BY_USER = {"default": 3600 * 24, "anonymous": 3600 * 24}  # 24hrs  # 24hrs

TILE_CACHE_TIMEOUT = 600  # seconds
CLUSTER_DISTANCE_MAX = 5000  # meters
GRAPH_MODEL_CACHE_TIMEOUT = None

OAUTH_CLIENT_ID = ""  #'9JCibwrWQ4hwuGn5fu2u1oRZSs9V6gK8Vu8hpRC4'

APP_TITLE = "Arches | Heritage Data Management"
COPYRIGHT_TEXT = "All Rights Reserved."
COPYRIGHT_YEAR = "2019"

ENABLE_CAPTCHA = False
# RECAPTCHA_PUBLIC_KEY = ''
# RECAPTCHA_PRIVATE_KEY = ''
# RECAPTCHA_USE_SSL = False
NOCAPTCHA = True
# RECAPTCHA_PROXY = 'http://127.0.0.1:8000'

# EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'  #<-- Only need to uncomment this for testing without an actual email server
# EMAIL_USE_TLS = True
# EMAIL_HOST = 'smtp.gmail.com'
EMAIL_HOST_USER = "xxxx@xxx.com"
# EMAIL_HOST_PASSWORD = 'xxxxxxx'
# EMAIL_PORT = 587

DEFAULT_FROM_EMAIL = EMAIL_HOST_USER

CELERY_BROKER_URL = ""  # RabbitMQ --> "amqp://guest:guest@localhost",  Redis --> "redis://localhost:6379/0"
CELERY_ACCEPT_CONTENT = ["json"]
CELERY_RESULT_BACKEND = "django-db"  # Use 'django-cache' if you want to use your cache as your backend
CELERY_TASK_SERIALIZER = "json"


CELERY_SEARCH_EXPORT_EXPIRES = 24 * 3600  # seconds
CELERY_SEARCH_EXPORT_CHECK = 3600  # seconds

CELERY_BEAT_SCHEDULE = {
    "delete-expired-search-export": {
        "task": "arches.app.tasks.delete_file",
        "schedule": CELERY_SEARCH_EXPORT_CHECK,
    },
    "notification": {
        "task": "arches.app.tasks.message",
        "schedule": CELERY_SEARCH_EXPORT_CHECK,
        "args": ("Celery Beat is Running",),
    },
}

# Set to True if you want to send celery tasks to the broker without being able to detect celery.
# This might be necessary if the worker pool is regulary fully active, with no idle workers, or if
# you need to run the celery task using solo pool (e.g. on Windows). You may need to provide another
# way of monitoring celery so you can detect the background task not being available.
CELERY_CHECK_ONLY_INSPECT_BROKER = False

CANTALOUPE_DIR = os.path.join(ROOT_DIR, UPLOADED_FILES_DIR)
CANTALOUPE_HTTP_ENDPOINT = "http://localhost:8182/"

ACCESSIBILITY_MODE = False

RENDERERS = [
    {
        "name": "imagereader",
        "title": "Image Reader",
        "description": "Displays most image file types",
        "id": "5e05aa2e-5db0-4922-8938-b4d2b7919733",
        "iconclass": "fa fa-camera",
        "component": "views/components/cards/file-renderers/imagereader",
        "ext": "",
        "type": "image/*",
        "exclude": "tif,tiff,psd",
    },
    {
        "name": "pdfreader",
        "title": "PDF Reader",
        "description": "Displays pdf files",
        "id": "09dec059-1ee8-4fbd-85dd-c0ab0428aa94",
        "iconclass": "fa fa-file",
        "component": "views/components/cards/file-renderers/pdfreader",
        "ext": "pdf",
        "type": "application/pdf",
        "exclude": "tif,tiff,psd",
    },
]

# By setting RESTRICT_MEDIA_ACCESS to True, media file requests outside of Arches will checked against nodegroup permissions.
RESTRICT_MEDIA_ACCESS = False

# By setting RESTRICT_CELERY_EXPORT_FOR_ANONYMOUS_USER to True, if the user is attempting
# to export search results above the SEARCH_EXPORT_IMMEDIATE_DOWNLOAD_THRESHOLD
# value and is not signed in with a user account then the request will not be allowed.
RESTRICT_CELERY_EXPORT_FOR_ANONYMOUS_USER = False

# Dictionary containing any additional context items for customising email templates
EXTRA_EMAIL_CONTEXT = {
    "salutation": _("Hi"),
    "expiration": (datetime.now() + timedelta(seconds=CELERY_SEARCH_EXPORT_EXPIRES)).strftime("%A, %d %B %Y"),
}

# see https://docs.djangoproject.com/en/1.9/topics/i18n/translation/#how-django-discovers-language-preference
# to see how LocaleMiddleware tries to determine the user's language preference
# (make sure to check your accept headers as they will override the LANGUAGE_CODE setting!)
# also see get_language_from_request in django.utils.translation.trans_real.py
# to see how the language code is derived in the actual code

####### TO GENERATE .PO FILES DO THE FOLLOWING ########
# run the following commands
# language codes used in the command should be in the form (which is slightly different
# form the form used in the LANGUAGE_CODE and LANGUAGES settings below):
# --local={countrycode}_{REGIONCODE} <-- countrycode is lowercase, regioncode is uppercase, also notice the underscore instead of hyphen
# commands to run (to generate files for "British English, German, and Spanish"):
# django-admin.py makemessages --ignore=env/* --local=de --local=en --local=en_GB --local=es  --extension=htm,py
# django-admin.py compilemessages


# default language of the application
# language code needs to be all lower case with the form:
# {langcode}-{regioncode} eg: en, en-gb ....
# a list of language codes can be found here http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = "en"

# list of languages to display in the language switcher,
# if left empty or with a single entry then the switch won't be displayed
# language codes need to be all lower case with the form:
# {langcode}-{regioncode} eg: en, en-gb ....
# a list of language codes can be found here http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGES = [
    #   ('de', _('German')),
    ("en", _("English")),
    #   ('en-gb', _('British English')),
    #   ('es', _('Spanish')),
]

# override this to permenantly display/hide the language switcher
SHOW_LANGUAGE_SWITCH = len(LANGUAGES) > 1

# try:
#     from .package_settings import *
# except ImportError:
#     try:
#         from package_settings import *
#     except ImportError as e:
#         pass

# try:
#     from .settings_local import *
# except ImportError as e:
#     try:
#         from settings_local import *
#     except ImportError as e:
#         pass



from django.db import transaction
from django.utils.decorators import method_decorator
from django.views.generic import View
from django.views.decorators.csrf import csrf_exempt

from arches.app.models.tile import Tile
from arches.app.models.resource import Resource
from arches.app.utils.response import JSONResponse
from arches.app.utils.betterJSONSerializer import JSONDeserializer


@method_decorator(csrf_exempt, name="dispatch")
class InstrumentInfoStepFormSaveView(View):
    def post(self, request):
        data = JSONDeserializer().deserialize(request.body)
        transaction_id = data.pop("transaction_id", None)
        resourceinstance_id = data.pop("resourceinstance_id", None)

        try:
            with transaction.atomic():
                if resourceinstance_id:
                    resource = Resource.objects.get(pk=resourceinstance_id)
                    resource.load_tiles()

                    Tile.objects.filter(pk__in=[tile.pk for tile in resource.tiles]).delete()

                    resource.tiles = []  # ensures outdated tiles are not saved
                else:
                    resource = Resource()
                    resource.graph_id = "615b11ee-c457-11e9-910c-a4d18cec433a"

                for key, tiledata in data.items():
                    tile = Tile(tiledata)
                    data[key] = tile
                    resource.tiles.append(tile)

                resource.save(user=request.user, transaction_id=transaction_id)
                return JSONResponse(data)
        except:
            return JSONResponse(status=500)



from arches.app.search.base_index import BaseIndex


class SampleIndex(BaseIndex):
    def prepare_index(self):
        self.index_metadata = {
            "mappings": {
                "properties": {
                    "tile_count": {"type": "keyword"},
                    "graph_id": {"type": "keyword"},
                }
            }
        }
        super(SampleIndex, self).prepare_index()

    def get_documents_to_index(self, resourceinstance, tiles):
        return (
            {"tile_count": len(tiles), "graph_id": resourceinstance.graph_id},
            str(resourceinstance.resourceinstanceid),
        )



"""
ARCHES - a program developed to inventory and manage immovable cultural heritage.
Copyright (C) 2013 J. Paul Getty Trust and World Monuments Fund

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import os
import json
import time
import re
import csv

from arches.app.models import models as archesmodels
from django.core.management.base import BaseCommand
from django.db import transaction
from arches.app.models.resource import Resource

from arches.app.models.models import TileModel
from arches.app.datatypes.datatypes import DataTypeFactory
from arches.app.search.search_engine_factory import SearchEngineInstance
from arches.app.models.system_settings import settings


try:
    graph_uuid_map = settings.MODEL_NAME_UUID_MAP
except:
    graph_uuid_map = {}


fieldre = re.compile("\* FieldId:(.*?)$", flags=re.DOTALL)
branchre = re.compile("\* BranchId:(.*?)$", flags=re.DOTALL)
notere = re.compile("# Arches Note:(.*?)$", flags=re.DOTALL)
generalre = re.compile("# General:(.*?)$", flags=re.DOTALL)
specificre = re.compile("# Specific:(.*?)$", flags=re.DOTALL)


class Command(BaseCommand):
    """
    Command for importing JSON-LD data into Arches
    """

    def add_arguments(self, parser):

        parser.add_argument("-s", "--source", default="node_descs.csv", action="store", dest="source", help="TSV file to read data from")
        parser.add_argument("-d", "--dest", default="node_descs.csv", action="store", dest="dest", help="TSV filename to write to")
        parser.add_argument("-o", "--operation", default="export", action="store", dest="op", help="One of: update, export")
        parser.add_argument("-g", "--graph", action="store", dest="model", help="slug or uuid for graph to export")

        parser.add_argument("--count", action="store_true", dest="count", help="include count of tiles in export")

    def handle(self, *args, **options):
        if options["op"] == "update":
            self.handle_update(options)
        elif options["op"] == "export":
            self.handle_export(options)
        elif options["op"] == "copy":
            self.handle_copy(options)
        else:
            print("Unknown operation. Available: update, export")

    def handle_update(self, options):
        template = """# Specific:
{spec}

# General:
{desc}

# Arches Note:
{note}

* BranchId: {bid}
* FieldId: {fid}
"""

        with open(options["source"]) as fh:
            reader = csv.DictReader(fh, delimiter="\t")
            for rec in reader:
                node = archesmodels.Node.objects.get(pk=rec["NodeId"])
                if rec["Node Name"]:
                    node.name = rec["Node Name"]
                desc = template.format(spec=rec["Specific"], desc=rec["General"], note=rec["Note"], bid=rec["BranchId"], fid=rec["FieldId"])
                node.description = desc
                node.save()

    def handle_copy(self, options):
        # Try to force insert the docs into the target database using
        # https://docs.djangoproject.com/en/2.2/topics/db/multi-db/

        self.forced_target_source = {
            "28a4ae07-c062-11e9-a11d-a4d18cec433a": "d515c3a8-bbd8-11ea-b6b2-3af9d3b32b71",
            "741d3687-bfb4-11e9-bcc1-a4d18cec433a": "292cf278-bb15-11ea-85a6-3af9d3b32b71",
            "83cf1bdc-bfb4-11e9-8f51-a4d18cec433a": "3a5bc0a6-bb15-11ea-85a6-3af9d3b32b71",
        }

        m = options["model"]
        if not m:
            # do all of them
            graphs = [x for x in archesmodels.GraphModel.objects.using("target").all()]
        else:
            graphid = graph_uuid_map.get(m, None)
            if not graphid:
                # Check slug
                try:
                    graph = archesmodels.GraphModel.objects.using("target").get(slug=m)
                except:
                    print(f"Couldn't find a model definition for {m}; skipping")
                    return
            else:
                try:
                    graph = archesmodels.GraphModel.objects.using("target").get(pk=graphid)
                except:
                    print(f"Couldn't find a model for {graphid}; skipping")
                    return
            graphs = [graph]

        graph_alignment = {
            "b6c819b8-99f6-11ea-a9b7-3af9d3b32b71": "1810d182-b4b5-11ea-84f7-3af9d3b32b71",  # Instrument
            "9519cb4f-b25b-11e9-8c7b-a4d18cec433a": "1810d182-b4b5-11ea-84f7-3af9d3b32b71",  # Physical Thing
            "f71f7b9c-b25b-11e9-901e-a4d18cec433a": "9ffb6fcc-b4b4-11ea-84f7-3af9d3b32b71",  # Person
            "07883c9e-b25c-11e9-975a-a4d18cec433a": "d6774bfc-b4b4-11ea-84f7-3af9d3b32b71",  # Group
            "1b210ef3-b25c-11e9-a037-a4d18cec433a": "bdba56bc-b4b5-11ea-84f7-3af9d3b32b71",  # Collection or Set
            "707cbd78-ca7a-11e9-990b-a4d18cec433a": "0044f7da-b4b6-11ea-84f7-3af9d3b32b71",  # Digital Resources
            "cc8ed633-b25b-11e9-a13a-a4d18cec433a": "f6e89030-b4b4-11ea-84f7-3af9d3b32b71",  # Place
            "31e6b222-066c-11ea-b592-acde48001122": "3d461890-b4b5-11ea-84f7-3af9d3b32b71",  # Provenance Activity
            "a7b1a7c5-b25b-11e9-8a4e-a4d18cec433a": "6dad61aa-b4b5-11ea-84f7-3af9d3b32b71",  # Textual Work
            "ba892214-b25b-11e9-bf3e-a4d18cec433a": "933ee880-b4b5-11ea-84f7-3af9d3b32b71",  # Visual Work
            "0b9235d9-ca85-11e9-9fa2-a4d18cec433a": "734d1558-bfad-11ea-a62b-3af9d3b32b71",  # Activity
            "615b11ee-c457-11e9-910c-a4d18cec433a": "18faa214-c2c3-11ea-af13-3af9d3b32b71",  # Observation
            "5bece219-c456-11e9-8dcd-a4d18cec433a": "2022f91c-c2c1-11ea-af13-3af9d3b32b71",  # Modification
        }

        for tgraph in graphs:
            tgname = tgraph.name
            tgid = tgraph.pk
            if tgname == "Arches System Settings":
                continue
            sgid = graph_alignment.get(str(tgid), None)
            if not sgid:
                continue
            sgraph = archesmodels.GraphModel.objects.using("default").get(pk=sgid)

            print(f"Processing {sgraph.name} into {tgraph.name}")

            s_topnode = archesmodels.Node.objects.using("default").filter(graph=sgraph, istopnode=True)[0]
            t_topnode = archesmodels.Node.objects.using("target").filter(graph=tgraph, istopnode=True)[0]
            self.align_branches(s_topnode, t_topnode)

    def align_branches(self, snode, tnode):
        s_edges = archesmodels.Edge.objects.using("default").filter(domainnode=snode.pk)
        t_edges = archesmodels.Edge.objects.using("target").filter(domainnode=tnode.pk)
        for te in t_edges:
            poss = []
            tn = te.rangenode

            snid = self.forced_target_source.get(str(tn.pk), None)
            if snid:
                sn = archesmodels.Node.objects.using("default").get(pk=snid)
                poss = [sn]
            else:
                # Search for it
                for se in s_edges:
                    sn = se.rangenode
                    if se.ontologyproperty == te.ontologyproperty and sn.ontologyclass == tn.ontologyclass:
                        sdt = sn.datatype
                        tdt = tn.datatype
                        if sdt == tdt:
                            poss.append(sn)
                        elif sdt.startswith("concept") and tdt.startswith("concept"):
                            # just concept/concept-list difference
                            poss.append(sn)
                        elif sdt.startswith("resource-instance") and tdt.startswith("resource-instance"):
                            # ditto
                            poss.append(sn)
                        # else discard as not a match

            if len(poss) == 1:
                # We have a match, copy the description
                matchn = poss[0]
                # print(f"Match {tn.name} <--> {matchn.name}")
                tn.description = matchn.description
                tn.save(using="target")
                self.align_branches(matchn, tn)
            elif not poss:
                # No match
                if not tn.name.strip().endswith("source"):
                    print(f'*** No match in source data graph for "{tn.name}"')
            else:
                # Multiple
                print(f"!!! Ambiguous match in source data graph for {tn.name}")

    def handle_export(self, options):
        m = options["model"]
        if not m:
            # do all of them
            graphs = [x for x in archesmodels.GraphModel.objects.filter()]
        else:
            graphid = graph_uuid_map.get(m, None)
            if not graphid:
                # Check slug
                try:
                    graph = archesmodels.GraphModel.objects.get(slug=m)
                except:
                    print(f"Couldn't find a model definition for {m}; skipping")
                    return
            else:
                try:
                    graph = archesmodels.GraphModel.objects.get(pk=graphid)
                except:
                    print(f"Couldn't find a model for {graphid}; skipping")
                    return
            graphs = [graph]

        print(f"Exporting Documentation Fields to {options['dest']}")

        with open(options["dest"], "w") as cfh:
            writer = csv.writer(
                cfh,
                delimiter="\t",
            )
            writer.writerow(
                (
                    "Graph",
                    "NodeId",
                    "Node Name",
                    "Specific",
                    "General",
                    "Note",
                    "BranchId",
                    "FieldId",
                    "Class",
                    "Property",
                    "DataType",
                    "Count",
                    "Path",
                )
            )

            for graph in graphs:
                gname = graph.name
                if gname == "Arches System Settings":
                    continue
                print(f"Processing {gname}...")
                nodes = archesmodels.Node.objects.filter(graph_id=graph.pk)
                for node in nodes:
                    name = node.name
                    desc = node.description
                    dfields = self.parse_description(desc)
                    # Now write line to a CSV
                    inbound = archesmodels.Edge.objects.filter(rangenode=node.pk)
                    if inbound:
                        prop = self.make_readable(inbound[0].ontologyproperty)
                    else:
                        prop = ""
                    path = self.get_inbound_path(node)

                    if options["count"] and graph.isresource:
                        c = archesmodels.TileModel.objects.filter(nodegroup_id=node.nodegroup.pk).count()
                    else:
                        c = -1
                    writer.writerow(
                        (
                            gname,
                            node.pk,
                            name,
                            dfields["specific"],
                            dfields["general"],
                            dfields["note"],
                            dfields["branchId"],
                            dfields["fieldId"],
                            self.make_readable(node.ontologyclass),
                            prop,
                            node.datatype,
                            c,
                            path,
                        )
                    )

    def get_inbound_path(self, node):
        plist = []
        inbound = archesmodels.Edge.objects.filter(rangenode=node.pk)
        while inbound:
            prop = self.make_readable(inbound[0].ontologyproperty)
            clss = self.make_readable(node.ontologyclass)
            plist.insert(0, f"{prop}[{clss}]")
            node = inbound[0].domainnode
            inbound = archesmodels.Edge.objects.filter(rangenode=node.pk)
        return "/".join(plist)

    def make_readable(self, prop):
        if prop:
            prop = prop.replace("http://www.cidoc-crm.org/cidoc-crm/", "crm:")
            prop = prop.replace("https://linked.art/ns/terms/", "la:")
            prop = prop.replace("http://www.w3.org/2001/XMLSchema#", "xsd:")
            prop = prop.replace("http://www.w3.org/2000/01/rdf-schema#", "rdfs:")
            prop = prop.replace("http://www.ics.forth.gr/isl/CRMdig/", "crmdig:")
            prop = prop.replace("http://www.ics.forth.gr/isl/CRMsci/", "crmsci:")
            prop = prop.replace("http://www.w3.org/2004/02/skos/core#", "skos:")
        else:
            prop = ""
        return prop

    def parse_description(self, desc):
        # Accept single string, return partitioned dict

        d = {"specific": "", "general": "", "note": "", "branchId": "", "fieldId": ""}

        if not desc:
            return d

        desc = desc.strip()
        if desc.startswith("# Specific:"):
            # Pop the fields off the end in turn
            re_order = [("fieldId", fieldre), ("branchId", branchre), ("note", notere), ("general", generalre), ("specific", specificre)]
            for name, regex in re_order:
                m = regex.search(desc)
                if m:
                    d[name] = m.group(1).strip()
                    desc = desc[: m.start()]
        else:
            d["specific"] = desc
        return d
